import gtk, gobject, cohoba
from cohoba.telepathy.Account import Account
from os.path import join
import cPickle

ACCOUNTS_FILE = join(cohoba.USER_COHOBA_DIR, "accounts.pickle")

class AccountList(gtk.ListStore):
	__gsignals__ = {
		"account-added" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [gobject.TYPE_PYOBJECT]),
		"account-deleted" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [gobject.TYPE_PYOBJECT]),
	}
		
	def __init__(self):
		gtk.ListStore.__init__(self, gobject.TYPE_PYOBJECT)
		self.accounts = {}
		
	def add_account(self, account=None):
		if account == None:
			account = Account()
		iter = self.append([account])
		self.accounts[account] = gtk.TreeRowReference(self, self.get_path(iter))
		account.connect('changed', self.on_account_changed)
		self.emit('account-added', account)
		return account
	
	def on_account_changed(self, account):
		if account not in self.accounts:
			return
			
		row = self.accounts[account]
		if row.get_path() == None:
			return
			
		path, iter = row.get_path(), self.get_iter(row.get_path())
	
		self.row_changed(path, iter)
		
	def remove_account(self, account):
		iter = self.get_iter(self.accounts[account].get_path())
		self.remove(iter)
		account.disconnect_account()
		del self.accounts[account]
		self.emit('account-deleted', account)

	def load(self):
		print 'Loading accounts'
		try:
			loaded = cPickle.load(file(ACCOUNTS_FILE))
			for saved in loaded:
				account = self.add_account()
				account.account, account.password, account.protocol, account.manager, account.auto_connect, account.save_pass = saved
			
				if account.auto_connect:
					account.connect_account()
		except Exception, e:
			print 'Could not load accounts file:', e
		
	def save(self):
		to_save = []
		print 'Saving accounts'
		for account in [row[0] for row in self]:
			password = None
			if account.save_pass:
				password = account.password
				
			to_save.append((account.account, password, account.protocol, account.manager, account.auto_connect, account.save_pass))
		
		cPickle.dump(to_save, file(ACCOUNTS_FILE, 'w'), cPickle.HIGHEST_PROTOCOL)
			
account_list = AccountList()
def get_account_list():
	return account_list
