from cohoba.telepathy.AccountList import get_account_list
from cohoba.Utils import global_error_handler
from cohoba.telepathy.MergedContactList import get_contact_list
from cohoba.ui.ConversationWindowManager import get_conversation_window_manager
from cohoba.telepathy.Constants import (
	CONNECTION_HANDLE_TYPE_LIST, CONNECTION_HANDLE_TYPE_CONTACT,
	CONNECTION_HANDLE_TYPE_NONE)
from cohoba.telepathy.DBusProxyProvider import get_proxy_provider
from cohoba.telepathy.TelepathyGroup import TelepathyGroup

CT_CONTACT_LIST = "org.freedesktop.Telepathy.Channel.Type.ContactList"
CT_STREAMED_MEDIA = "org.freedesktop.Telepathy.Channel.Type.StreamedMedia"
CT_TEXT = "org.freedesktop.Telepathy.Channel.Type.Text"

class ChannelHandler:
	def __init__(self):
		self.connected_accounts = []

	def start(self):
		try:
			get_account_list().load()
		except Exception, e:
			print e
			
		for row in get_account_list():
			row[0].connect('changed', self.on_account_changed)
		get_account_list().connect("account-added", self.on_account_added)
		get_account_list().connect("account-deleted", self.on_account_deleted)
		
		get_proxy_provider().create_proxy(
			"org.freedesktop.Telepathy.VoipEngine",
			"/org/freedesktop/Telepathy/VoipEngine")
		self.voip_engine_iface = get_proxy_provider().get_iface(
			"/org/freedesktop/Telepathy/VoipEngine",
			"org.freedesktop.Telepathy.ChannelHandler")
	
	def stop(self):
		for row in get_account_list():
			row[0].disconnect_account()

	def request_channel(self, conn_path, type, handle_type, handle,
			suppress_handler, **args):

		if 'reply_handler' not in args:
			args['reply_handler'] = lambda *args: None

		if 'error_handler' not in args:
			args['error_handler'] = global_error_handler

		conn_iface = get_proxy_provider().get_iface(conn_path,
			'org.freedesktop.Telepathy.Connection')
		return conn_iface.RequestChannel(type, handle_type, handle,
			suppress_handler, **args)

	def start_conversation(self, contact):
		try:
			get_conversation_window_manager().show_existing(contact.conn_path,
				contact.handle)
		except:
			self.request_channel(
				contact.conn_path, CT_TEXT, CONNECTION_HANDLE_TYPE_CONTACT,
				contact.handle, False)

	def _media_badger(self, channel, contact):
		print channel
		window = get_conversation_window_manager().show_existing(
			contact.conn_path, contact.handle)
		print (contact.conn_path, channel)
		media = TelepathyGroup(contact.conn_path, channel, 0)
		window.start_media(media)

	def start_media(self, contact):
		#conn_iface = get_proxy_provider().get_iface(contact.conn_path,
		#	'org.freedesktop.Telepathy.Connection')
		#conn_iface.RequestChannel(
		#	"org.freedesktop.Telepathy.Channel.Type.StreamedMedia",
		#	CONNECTION_HANDLE_TYPE_CONTACT, contact.handle, False,
		#	reply_handler=lambda *args: None, error_handler=global_error_handler)
		print self, 'start_media!'
		obj = self.request_channel(
			contact.conn_path, CT_STREAMED_MEDIA,
			CONNECTION_HANDLE_TYPE_CONTACT, contact.handle, True,
			reply_handler=lambda channel: self._media_badger(channel, contact))


	def on_account_added(self, account_list, account):
		account.connect('changed', self.on_account_changed)

	def on_account_deleted(self, account_list, account):
		if account in self.connected_accounts:
			self.connected_accounts.remove(account)

	def on_account_changed(self, account):
		if not account.is_connected and account in self.connected_accounts:
			self.connected_accounts.remove(account)

		if account.is_connected and account not in self.connected_accounts:
			account.conn_iface.connect_to_signal('NewChannel',
				lambda *args: self.on_new_channel(account, *args))
			account.conn_iface.ListChannels(reply_handler=
				lambda *args: self.on_list_channels(account, *args),
					error_handler=global_error_handler)
			self.connected_accounts.append(account)
	
	def on_list_channels(self, account, channels):
		for channel in channels:
			self.on_new_channel(account, *channel)

	def on_new_channel(self, account, chan_obj, chan_type, handle_type,
			handle, suppress_handler=False):
		print ('New Channel: ', chan_obj, chan_type, handle_type, handle,
			suppress_handler)

		if suppress_handler:
			print ('** Handler suppressed:', chan_obj, chan_type, handle_type,
				handle, suppress_handler)
			#return

		handled = False

		if handle_type != CONNECTION_HANDLE_TYPE_NONE:
			name = account.conn_iface.InspectHandle(handle_type, handle)
		else:
			name = None

		if (chan_type == CT_CONTACT_LIST and
				handle_type == CONNECTION_HANDLE_TYPE_LIST and
				name == "subscribe"):
			get_contact_list().add_group(account.conn_obj_path, chan_obj,
				handle)
			handled = True

		elif (chan_type == CT_TEXT and
				handle_type == CONNECTION_HANDLE_TYPE_CONTACT):
			get_conversation_window_manager().show(
				account.conn_obj_path, chan_obj, handle)
			handled = True

		elif chan_type == CT_STREAMED_MEDIA:
			print (account.conn_obj_path, chan_obj, handle)

			media = TelepathyGroup(account.conn_obj_path, chan_obj, handle)

			if handle:
				window = get_conversation_window_manager().show_existing(
					account.conn_obj_path, handle)
				window.start_media(media)

			self.voip_engine_iface.HandleChannel(
				get_proxy_provider().get_service(account.conn_obj_path),
					account.conn_obj_path, chan_type, chan_obj, handle_type,
					handle, reply_handler=lambda: None,
					error_handler=global_error_handler)

			handled = True

		if not handled:
			print ('** New Channel Not handled', chan_obj, chan_type,
				handle_type, handle, suppress_handler)

channel_handler = ChannelHandler()
def get_channel_handler():
	return channel_handler
