import gtk, gobject, dbus, cohoba
from gettext import gettext as _
from os.path import join
import cPickle

from cohoba.Utils import global_error_handler
from cohoba.telepathy.Contact import Contact#get_contact
from cohoba.telepathy.ContactGroup import ContactGroup
from cohoba.telepathy.TelepathyGroup import TelepathyGroup
from cohoba.telepathy.TelepathyGroupMember import GROUP_MEMBER_FLAG_MEMBER, GROUP_MEMBER_FLAG_LOCAL, GROUP_MEMBER_FLAG_REMOTE
from cohoba.ui.PresenceIcons import get_presence_icons

GROUPS_FILE = join(cohoba.USER_COHOBA_DIR, "groups.pickle")
	
ICON_THEME = gtk.icon_theme_get_default()
class MergedContactList(gtk.ListStore):
	COL_CONTACT = 0
	COL_STRING = 1
	COL_PIXBUF = 2
	
	PERSON_ICON = ICON_THEME.load_icon("stock_person", 48, gtk.ICON_LOOKUP_USE_BUILTIN)
	PEOPLE_ICON = ICON_THEME.load_icon("stock_people", 48, gtk.ICON_LOOKUP_USE_BUILTIN)
	
	def __init__(self):
		gtk.ListStore.__init__(self, gobject.TYPE_PYOBJECT, gobject.TYPE_STRING, gtk.gdk.Pixbuf)
		self.groups = {}
		self.load_contact_groups()
	
	# --- Contact Group related methods ---
	def load_contact_groups(self):
		print 'Loading contact groups'
		try:
			contact_groups = cPickle.load(file(GROUPS_FILE))
			for name, groups in contact_groups:
				self.add_contact_group(name, groups)				
		except IOError, e:
			print 'Could not load contact groups:', e
		
		# Adding uncategorized, if not existing
		uncategorized_found = False
		for group in [row[0] for row in self if row[0].__class__ == ContactGroup]:
			if group.groups != None and len(group.groups) == 0:
				uncategorized_found = True
				break
		
		if not uncategorized_found:
			self.add_contact_group(_("Uncategorized"), [])
			
	def save_contact_groups(self):
		print 'Saving contact groups'
		save = []
		for group in [row[0] for row in self if row[0].__class__ == ContactGroup]:
			save.append((group.name, group.groups))
		cPickle.dump(save, file(GROUPS_FILE, 'w'), cPickle.HIGHEST_PROTOCOL)
	
	def add_contact_group(self, name, groups):
		group = ContactGroup(name, groups)
		self.append([group, self.get_contact_group_string(group), self.PEOPLE_ICON])
		self.save_contact_groups()
	# ---
	
	# --- Public interface for contacts ---
	def add_group(self, conn_path, obj_path, handle):
		if conn_path in self.groups:
			return
			
		self.groups[conn_path] = TelepathyGroup(conn_path, obj_path, handle)
		self.groups[conn_path]._member_for_data = self._member_for_data
		self.groups[conn_path]._add_member = lambda handle, flag: self._add_member(conn_path, handle, flag)
		self.groups[conn_path]._remove_member = self._remove_member
		self.groups[conn_path]._channel_closed = lambda: self._channel_closed(conn_path)
	
	def add_member(self, conn_path, name, reason="No Reason."):
		print 'Trying to add:', conn_path, name, reason
		return self.groups[conn_path].add_member(name, reason="No Reason.")
	
	def remove_member(self, conn_path, handle, reason="No Reason."):
		self.groups[conn_path].remove_member(handle, reason="No Reason.")
	
	def get_contact(self, conn_path, handle):
		contact = [row[0] for row in self if row[0].__class__ == Contact and row[0].conn_path == conn_path and row[0].handle == handle]
		if len(contact) >= 1:
			return contact[0]
		
	# --- 
	
	# --- Abstract methods from TelepathyGroup ---
	def _member_for_data(self, row):
		return self[row.get_path()][self.COL_CONTACT]
			
	def _add_member(self, conn_path, handle, flag):
		print 'Adding member:', conn_path, handle
		contact=Contact(conn_path, handle)
		contact.set_flag(flag)
		iter = self.append([contact, self.get_contact_string(contact), self.get_contact_pixbuf(contact)])
		contact.connect('updated', self.on_contact_updated, conn_path)
		
		return gtk.TreeRowReference(self, self.get_path(iter))
		
	def _remove_member(self, row):
		path, iter = row.get_path(), self.get_iter(row.get_path())
		self.remove(iter)
	
	def _channel_closed(self, conn_path):
		del self.groups[conn_path]
		
	def on_contact_updated(self, contact, conn_path):
		row = self.groups[conn_path].members[contact.handle]
		path, iter = row.get_path(), self.get_iter(row.get_path())

		self[iter][self.COL_STRING] = self.get_contact_string(contact)
		self[iter][self.COL_PIXBUF] = self.get_contact_pixbuf(contact)
		
		self.row_changed(path, iter)
			
	def __repr__(self):
		return 'Contact List: %s' % ([row[self.COL_CONTACT] for row in self],)
	# --- 
	
	# --- Repreentation of Contacts and Groups
	def get_contact_string(self, contact):
		status_msg = ""
		if contact.status_message != None:
			status_msg = "\n"+contact.status_message
			
		name = contact.name
		if contact.alias != None:
			name = "%s\n%s" % (contact.alias, name)
			
		string = '<b>%s</b>\n<span size="small">%s%s</span>' % (name, contact.status, status_msg)
		if contact.flag == GROUP_MEMBER_FLAG_LOCAL:
			string += "\n<span size='small'><i>%s</i></span>" % "Locally pending"
		elif contact.flag == GROUP_MEMBER_FLAG_REMOTE:
			string += "\n<span size='small'><i>%s</i></span>" % "Waiting remote authorization"
		
		string += "\n<span size='small'><i>%s</i></span>" % ','.join(contact.groups)
		
		return string
		
	def get_contact_pixbuf(self, contact):
		return get_presence_icons(contact.status_code)[1]
	
	def get_contact_group_string(self, group):
		return '<b>%s</b>\n<span size="small">%s</span>' % (group.name, group.groups)
	# ---
	
	
contact_list = MergedContactList()
def get_contact_list():
	return contact_list
	
