import gobject, dbus

from cohoba.Utils import global_error_handler
from cohoba.telepathy.Constants import CONNECTION_HANDLE_TYPE_CONTACT
from cohoba.telepathy.TelepathyGroupMember import TelepathyGroupMember, GROUP_MEMBER_FLAG_LOCAL, GROUP_MEMBER_FLAG_REMOTE, GROUP_MEMBER_FLAG_MEMBER
from cohoba.telepathy.DBusProxyProvider import get_proxy_provider

class TelepathyGroup:
	def __init__(self, conn_path, obj_path, handle):
		self.handle = handle
		self.members = {}
		self.member_list = {}
		
		self.conn_path = conn_path
		self.conn_iface = get_proxy_provider().get_iface(conn_path, 'org.freedesktop.Telepathy.Connection')
		
		get_proxy_provider().create_proxy(get_proxy_provider().get_service(conn_path), obj_path)
		
		self.chan_iface = get_proxy_provider().get_iface(obj_path, 'org.freedesktop.Telepathy.Channel')
		self.chan_iface.connect_to_signal('Closed', self.on_channel_closed)
		
		self.member_list_iface = get_proxy_provider().get_iface(obj_path, 'org.freedesktop.Telepathy.Channel.Interface.Group')
		self.member_list_iface.connect_to_signal('MembersChanged', self.on_members_changed)
		self.member_list_iface.GetMembers(reply_handler=self.on_got_members, error_handler=global_error_handler)
	
	# --------External methods----------------
	def add_member(self, name, reason="No Reason."):
		handle = self.conn_iface.RequestHandle(CONNECTION_HANDLE_TYPE_CONTACT, name)
		print 'Got new handle:', handle
		self.member_list_iface.AddMembers([handle], reason, reply_handler=lambda: None, error_handler=global_error_handler)
		return handle
	
	def remove_member(self, handle, reason="No Reason."):
		self.member_list_iface.RemoveMembers([handle], reason, reply_handler=lambda: None, error_handler=global_error_handler)
	#--------------------------------------------------
		
	def add_members(self, members, flag=GROUP_MEMBER_FLAG_MEMBER):
		for handle in members:
			if handle in self.members:
				member = self._member_for_data(self.members[handle])
				member.set_flag(flag)
			else:
				self.members[handle] = self._add_member(handle, flag)
			
	def remove_members(self, members):
		for handle in members:
			if handle not in self.members:
				continue
				
			self._remove_member(self.members[handle])
			del self.members[handle]
			
	def on_members_changed(self, reason, added, removed, local_pending, remote_pending):
		print 'Members changed:', reason, added, removed, local_pending, remote_pending
		self.add_members(local_pending, GROUP_MEMBER_FLAG_LOCAL)
		self.add_members(remote_pending, GROUP_MEMBER_FLAG_REMOTE)
		self.add_members(added)
		self.remove_members(removed)
		print [self._member_for_data(self.members[handle]) for handle in self.members.keys()]
		
	def on_got_members(self, members):
		print 'Got Members:', members
		self.add_members(members)
		print [self._member_for_data(self.members[handle]) for handle in self.members.keys()]
	
	def on_channel_closed(self):
		print 'Channel Closed'
		self.remove_members(self.members.keys())
		self._channel_closed()
		
	def _member_for_data(self, handle):
		return self.member_list[handle]
				
	def _add_member(self, handle, flag):
		member = TelepathyGroupMember(handle)
		member.set_flag(flag)
		self.member_list[handle] = member
		return handle
		
	def _remove_member(self, handle):
		del self.member_list[handle]
	
	def _channel_closed(self):
		pass
