import gtk, gtk.glade, gobject
from os.path import join
import cohoba

from cohoba.ui.ProtocolComboBox import ProtocolComboBox

class AccountEditor(gtk.Window):
	def __init__(self, account):
		gtk.Window.__init__(self)
		
		self.account = account
		
		self.set_property("border_width", 12)
		self.set_property("window_position", gtk.WIN_POS_CENTER)
		
		glade = gtk.glade.XML(join(cohoba.SHARED_DATA_DIR, "accounts.glade"), "account_modification_contents")
		self.add(glade.get_widget("account_modification_contents"))
		
		self.account_entry = glade.get_widget("account")
		self.password = glade.get_widget("password")
		self.save_password = glade.get_widget("save_password")
		self.close = glade.get_widget("close")
		
		self.protocol = ProtocolComboBox()
		glade.get_widget("table").attach(self.protocol, 1, 2, 0, 1, xoptions=gtk.FILL, yoptions=gtk.FILL)
		self.protocol.show()
		
		self.connect('destroy', self.on_close)
		self.close.connect("clicked", lambda widget: self.destroy())
		from cohoba.telepathy.AccountList import get_account_list
		get_account_list().connect('account-deleted', self.on_account_deleted)
		self.sync_account()

	def sync_account(self):
		self.protocol.set_active_protocol(self.account.protocol, self.account.manager)
		if self.account.account != None:
			self.account_entry.set_text(self.account.account)
		if self.account.password != None:
			self.password.set_text(self.account.password)
		self.save_password.set_active(self.account.save_pass)
		
	def on_account_deleted(self, account_list, deleted_account):
		if deleted_account == self.account:
			self.destroy()
			
	def on_close(self, *args):
		active_proto = self.protocol.get_active_protocol()
		if active_proto != None:
			proto, manager = active_proto
			self.account.protocol = proto
			self.account.manager = manager
		
		if self.account_entry.get_text() == "":
			self.account.account = None
		else:
			self.account.account = self.account_entry.get_text()
			
		if self.password.get_text() == "":
			self.account.password = None
		else:
			self.account.password = self.password.get_text()
			
		self.account.save_pass = self.save_password.get_active()
		
		self.account.changed()

account_editors = {}
def get_editor_for_account(account):
	if account not in account_editors:
		account_editors[account] = AccountEditor(account)
		def on_account_deleted(account):
			if account in account_editors:
				del account_editors[account]
			
		account_editors[account].connect('destroy', lambda win: on_account_deleted(account))
	
	return account_editors[account]
