import gtk, gobject
from cohoba.Utils import load_icon
from cohoba.telepathy.AccountList import get_account_list
from cohoba.Utils import global_error_handler
from cohoba.telepathy.DBusProxyProvider import get_proxy_provider
from cohoba.ui.PresenceIcons import get_presence_icons

class PresenceComboBoxModel(gtk.ListStore):
	COL_PIXBUF = 0
	COL_TEXT = 1
	COL_NAME = 2
	
	def __init__(self):
		gtk.ListStore.__init__(self, gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING)
				
		presences = [
			("available", "Available"),
			("away", "Away"),
#			("brb", "Be Right Back"),
#			("busy", "Busy"),
			("dnd", "Do Not Disturb"),
			("xa", "Extended Away"),
			("hidden", "Invisible"),
			("offline", "Offline"),
		]
		
		for name, text in presences:
			self.append([get_presence_icons(name)[0], text, name])
				
class PresenceComboBox(gtk.ComboBox):
	
	__gsignals__ = {
		"presence": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [gobject.TYPE_STRING, gobject.TYPE_STRING]),
	}
	
	def __init__(self, message_entry):
		gtk.ComboBox.__init__(self, PresenceComboBoxModel())
		
		self.message_entry = message_entry
		
		cell = gtk.CellRendererPixbuf()
		self.pack_start(cell, False)
		self.add_attribute(cell, 'pixbuf', 0)
		
		cell = gtk.CellRendererText()
		self.pack_start(cell, True)
		self.add_attribute(cell, 'text', 1)
		
		self.connect('changed', self.on_presence_change)
		self.message_entry.connect('activate', self.on_presence_change)
		self.message_entry.connect('focus-out-event', self.on_presence_change)
		
	def on_presence_change(self, widget, event=None):
		name = self.get_model()[self.get_active_iter()][PresenceComboBoxModel.COL_NAME]
		message = self.message_entry.get_text()
		
		for account in [row[0] for row in get_account_list() if row[0].is_connected]:
			presence_iface = get_proxy_provider().get_iface(account.conn_obj_path, 'org.freedesktop.Telepathy.Connection.Interface.Presence')
			presence_iface.SetStatus({name: {"message": message}}, reply_handler=lambda: None, error_handler=global_error_handler)
		
